/* Copyright © 2012 Matthew Champion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of mattunderscore.com nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MATTHEW CHAMPION BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

package com.mattunderscore.filter;

import java.io.IOException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.ServletContext;

/**
 * An abstract generic filter that provides a boilerplate implementation of
 * {@link Filter#init(FilterConfig)}. It sets the {@link FilterConfig} and {@link ServletContext} objects.
 * 
 * More complex configuration of the filter can be implemented by overriding the
 * configureFilter method. This is executed by {@link #init(FilterConfig)} method after the
 * {@link FilterConfig} and {@link ServletContext} have been set.
 * 
 * @author Matt Champion
 * @since 0.0.13
 */
public abstract class GenericFilter implements Filter
{

    /**
     * The {@link FilterConfig} set by {@link #setFilterConfig(FilterConfig)}.
     * @since 0.0.13
     */
    protected FilterConfig filterConfig;

    /**
     * The servlet context of the filter config set by {@link #setFilterConfig(FilterConfig)}.
     * @since 0.0.13
     */
    protected ServletContext context;

    public abstract void doFilter(ServletRequest request,
            ServletResponse response, FilterChain chain) throws IOException,
            ServletException;

    /**
     * Return the filter configuration object for this filter.
     * 
     * @return The {@link FilterConfig} that was passed in through {@link #init(FilterConfig)}.
     * @since 0.0.13
     */
    public final FilterConfig getFilterConfig()
    {
        return filterConfig;
    }

    /**
     * Set the filter configuration object for this filter.
     * 
     * @param filterConfig
     *            The filter configuration object
     * @since 0.0.13
     */
    public final void setFilterConfig(final FilterConfig filterConfig)
    {
        this.context = filterConfig.getServletContext();
        this.filterConfig = filterConfig;
    }

    /**
     * Callback method for configuring the filter.
     * @since 0.0.13
     */
    public void configureFilter()
    {
    }

    @Override
    public void destroy()
    {
    }

    @Override
    public final void init(FilterConfig filterConfig)
    {
        setFilterConfig(filterConfig);
        configureFilter();
    }
}
